/***********************************************************
 By using this material you agree to comply with the terms
 governing the MBTI systems and with the MBTI Site Terms and
 Conditions available for view at http://www.mbtrading.com/
 Copyright 2003 MB Trading, Inc. ALL RIGHTS RESERVED.
************************************************************/

// QuoteSink.h: interface for the CQuoteSink class.

#if !defined(AFX_QUOTESINK_H__4FA5A325_C89C_46FF_AC69_BEE969008C06__INCLUDED_)
#define AFX_QUOTESINK_H__4FA5A325_C89C_46FF_AC69_BEE969008C06__INCLUDED_

#if _MSC_VER > 1000
#pragma once
#endif // _MSC_VER > 1000

// Internal id for event sink
#define ID_QUOTESEVENTS		1000

//////////////////////////////////////////////////////////////////////////
// Macros for implementing IMbtQuotesNotify direct callbacks in CCmdTargets (ie CDerived)

// *** In the .h **
//    Add the following member variable:
//	  IMbtQuotesNotifyPtr	m_pQNotify;
//
//    Make sure to add DECLARE_INTERFACE_MAP(), then add the following (public:)
//    to create a nested m_xMbtQuotesNotify.

// This will setup the nested m_xMbtQuotesNotify
#define DECLARE_MBTQUOTESNOTIFY_INTERFACE_PART()\
    BEGIN_INTERFACE_PART(MbtQuotesNotify, IMbtQuotesNotify)\
         STDMETHOD(raw_OnQuoteData)(QUOTERECORD* pRec);\
         STDMETHOD(raw_OnLevel2Data)(LEVEL2RECORD* pRec);\
         STDMETHOD(raw_OnTSData)(TSRECORD* pRec);\
         STDMETHOD(raw_OnOptionsData)(OPTIONSRECORD* pRec);\
    END_INTERFACE_PART(MbtQuotesNotify);


// *** In the .cpp ***
// Add the following:
//BEGIN_INTERFACE_MAP(CDerived, CBase)
//		INTERFACE_PART(CDerived, __uuidof(IMbtQuotesNotify), MbtQuotesNotify)
//END_INTERFACE_MAP()

// Add to CDerive'd ctor:
//	// Get our callback interface
//	EnableAutomation();
//	LPDISPATCH pDisp = GetIDispatch(FALSE); // No addref here
//	m_pQNotify = pDisp;						// Automatic QI by smart pointer


// If you only want to sink ONE particular event, use one of the following four
// macros and implement the member function noted:

// L1 (aka Quotes) handler only. class_name needs to implement the following function:
// HRESULT OnQuoteData(QUOTERECORD* pRec);
#define IMPLEMENT_MBTQUOTESNOTIFY_L1_SINK(class_name)\
	IMPLEMENT_MBTQUOTESNOTIFY_IUNKNOWNS(class_name)\
	IMPLEMENT_MBTQUOTESNOTIFY_NOTIMPL(class_name, OnLevel2Data, LEVEL2RECORD*)\
	IMPLEMENT_MBTQUOTESNOTIFY_NOTIMPL(class_name, OnTSData,		TSRECORD*)\
	IMPLEMENT_MBTQUOTESNOTIFY_NOTIMPL(class_name, OnOptionsData,OPTIONSRECORD*)\
	IMPLEMENT_MBTQUOTESNOTIFY(class_name, OnQuoteData, QUOTERECORD*)

// L2 handler only. class_name needs to implement the following function:
// HRESULT OnLevel2Data(LEVEL2RECORD* pRec);
#define IMPLEMENT_MBTQUOTESNOTIFY_L2_SINK(class_name)\
	IMPLEMENT_MBTQUOTESNOTIFY_IUNKNOWNS(class_name)\
	IMPLEMENT_MBTQUOTESNOTIFY_NOTIMPL(class_name, OnTSData,		TSRECORD*,)\
	IMPLEMENT_MBTQUOTESNOTIFY_NOTIMPL(class_name, OnOptionsData,OPTIONSRECORD*)\
	IMPLEMENT_MBTQUOTESNOTIFY_NOTIMPL(class_name, OnQuoteData,	QUOTERECORD*)\
	IMPLEMENT_MBTQUOTESNOTIFY(class_name, OnLevel2Data, LEVEL2RECORD*)

// TS handler only. class_name needs to implement the following function:
// HRESULT OnTSData(TSRECORD* pRec);
#define IMPLEMENT_MBTQUOTESNOTIFY_TS_SINK(class_name)\
	IMPLEMENT_MBTQUOTESNOTIFY_IUNKNOWNS(class_name)\
	IMPLEMENT_MBTQUOTESNOTIFY_NOTIMPL(class_name, OnLevel2Data, LEVEL2RECORD*)\
	IMPLEMENT_MBTQUOTESNOTIFY_NOTIMPL(class_name, OnOptionsData,OPTIONSRECORD*)\
	IMPLEMENT_MBTQUOTESNOTIFY_NOTIMPL(class_name, OnQuoteData,	QUOTERECORD*)\
	IMPLEMENT_MBTQUOTESNOTIFY(class_name, OnTSData, TSRECORD*)

// Options handler only. class_name needs to implement the following function:
// HRESULT OnOptionsData(OPTIONSRECORD* pRec);
#define IMPLEMENT_MBTQUOTESNOTIFY_OPTIONS_SINK(class_name)\
	IMPLEMENT_MBTQUOTESNOTIFY_IUNKNOWNS(class_name)\
	IMPLEMENT_MBTQUOTESNOTIFY_NOTIMPL(class_name, OnLevel2Data, LEVEL2RECORD*)\
	IMPLEMENT_MBTQUOTESNOTIFY_NOTIMPL(class_name, OnTSData,		TSRECORD*)\
	IMPLEMENT_MBTQUOTESNOTIFY_NOTIMPL(class_name, OnQuoteData,	QUOTERECORD*)\
	IMPLEMENT_MBTQUOTESNOTIFY(class_name, OnOptionsData, OPTIONSRECORD*)

// L1 AND L2 handlers. class_name needs to implement the following functions:
// HRESULT OnQuoteData(QUOTERECORD* pRec);
// HRESULT OnLevel2Data(LEVEL2RECORD* pRec);
#define IMPLEMENT_MBTQUOTESNOTIFY_L1_L2_SINK(class_name)\
	IMPLEMENT_MBTQUOTESNOTIFY_IUNKNOWNS(class_name)\
	IMPLEMENT_MBTQUOTESNOTIFY_NOTIMPL(class_name, OnTSData,		TSRECORD*)\
	IMPLEMENT_MBTQUOTESNOTIFY_NOTIMPL(class_name, OnOptionsData,OPTIONSRECORD*)\
	IMPLEMENT_MBTQUOTESNOTIFY(class_name, OnQuoteData, QUOTERECORD*)\
	IMPLEMENT_MBTQUOTESNOTIFY(class_name, OnLevel2Data, LEVEL2RECORD*)

// L1 AND TS handlers. class_name needs to implement the following functions:
// HRESULT OnQuoteData(QUOTERECORD* pRec);
// HRESULT OnTSData(TSRECORD* pRec);
#define IMPLEMENT_MBTQUOTESNOTIFY_L1_TS_SINK(class_name)\
	IMPLEMENT_MBTQUOTESNOTIFY_IUNKNOWNS(class_name)\
	IMPLEMENT_MBTQUOTESNOTIFY_NOTIMPL(class_name, OnLevel2Data, LEVEL2RECORD*)\
	IMPLEMENT_MBTQUOTESNOTIFY_NOTIMPL(class_name, OnOptionsData,OPTIONSRECORD*)\
	IMPLEMENT_MBTQUOTESNOTIFY(class_name, OnQuoteData, QUOTERECORD*)\
	IMPLEMENT_MBTQUOTESNOTIFY(class_name, OnTSData, TSRECORD*)

// If you want to sink ALL events, use this one (not sure why you would want to
// except for demonstration purposes only).
// class_name needs to implement the following functions:
// HRESULT OnQuoteData(QUOTERECORD* pRec);
// HRESULT OnLevel2Data(LEVEL2RECORD* pRec);
// HRESULT OnTSData(TSRECORD* pRec);
// HRESULT OnOptionsData(OPTIONSRECORD* pRec);
#define IMPLEMENT_MBTQUOTESNOTIFY_ALL_SINK(class_name)\
	IMPLEMENT_MBTQUOTESNOTIFY_IUNKNOWNS(class_name)\
	IMPLEMENT_MBTQUOTESNOTIFY(class_name, OnQuoteData, QUOTERECORD*)\
	IMPLEMENT_MBTQUOTESNOTIFY(class_name, OnLevel2Data, LEVEL2RECORD*)\
	IMPLEMENT_MBTQUOTESNOTIFY(class_name, OnTSData, TSRECORD*)\
	IMPLEMENT_MBTQUOTESNOTIFY(class_name, OnOptionsData, OPTIONSRECORD*)


// If you need to implement multiple sinks, use these macros directly in the cpp.
// You must use the IMPLEMENT_MBTQUOTESNOTIFY_IUNKNOWNS, followed by
// IMPLEMENT_MBTQUOTESNOTIFY (if you WANT the event) or *NOTIMPL (if you
// DONT want the event - for each of the 4 events
#define IMPLEMENT_MBTQUOTESNOTIFY_IUNKNOWNS(class_name)\
	STDMETHODIMP_(ULONG) class_name::XMbtQuotesNotify::AddRef()\
	{\
		METHOD_PROLOGUE(class_name, MbtQuotesNotify)\
		return pThis->ExternalAddRef();\
	}\
	STDMETHODIMP_(ULONG) class_name::XMbtQuotesNotify::Release()\
	{\
		METHOD_PROLOGUE(class_name, MbtQuotesNotify)\
		return pThis->ExternalRelease();\
	}\
	STDMETHODIMP class_name::XMbtQuotesNotify::QueryInterface(REFIID iid, LPVOID* ppvObj)\
	{\
		METHOD_PROLOGUE(class_name, MbtQuotesNotify)\
		return pThis->ExternalQueryInterface(&iid, ppvObj);\
	}

#define IMPLEMENT_MBTQUOTESNOTIFY_NOTIMPL(class_name, function_name, param_type)\
	STDMETHODIMP class_name::XMbtQuotesNotify::raw_##function_name(param_type pRec)	{ return E_NOTIMPL; }

#define IMPLEMENT_MBTQUOTESNOTIFY(class_name, function_name, param_type)\
	STDMETHODIMP class_name::XMbtQuotesNotify::raw_##function_name(param_type pRec)\
	{\
		METHOD_PROLOGUE(class_name, MbtQuotesNotify)\
		/*Thunk this back to the containing parent class (which is a virtual\
		  function that can be overridden in derived classes to handle)*/\
		return pThis->function_name(pRec);\
	}

#endif // !defined(AFX_QUOTESINK_H__4FA5A325_C89C_46FF_AC69_BEE969008C06__INCLUDED_)