/***********************************************************
 By using this material you agree to comply with the terms
 governing the MBTI systems and with the MBTI Site Terms and
 Conditions available for view at http://www.mbtrading.com/
 Copyright 2003 MB Trading, Inc. ALL RIGHTS RESERVED.
************************************************************/

// CConsoleQuotes.cpp

#include "stdafx.h"
#include "CConsoleQuotes.h"
#include <winerror.h>

IMPLEMENT_DYNAMIC(CConsoleQuotes, CCmdTarget)

BEGIN_INTERFACE_MAP(CConsoleQuotes, CCmdTarget)
		INTERFACE_PART(CConsoleQuotes, __uuidof(IMbtQuotesNotify), MbtQuotesNotify)
END_INTERFACE_MAP()

IMPLEMENT_MBTQUOTESNOTIFY_ALL_SINK(CConsoleQuotes)

// CConsoleQuotes ctor, dtor
//////////////////////////////////////////////////////////////////////////

CConsoleQuotes::CConsoleQuotes(IMbtQuotesPtr& pQuotes)
	: CCmdTarget()
{
	ASSERT( NULL != pQuotes );
	m_pQuotes = pQuotes;

	EnableAutomation();      // Enable OLE automation
	LPDISPATCH pDisp = GetIDispatch(FALSE); // No addref here
	m_pQNotify = pDisp;      // Automatic QI by smart pointer
}

CConsoleQuotes::~CConsoleQuotes()
{
	// IMPORTANT - You must ALWAYS UnadviseAll during clean up.
	m_pQuotes->UnadviseAll(m_pQNotify);
	// m_pQNotify and m_pQuotes will auto-Release here...
}

bool CConsoleQuotes::DoAdvise(LPCTSTR pszSymbol)
{
	printf("Try AdviseSymbol() %s w/ level %d quotes...\n", pszSymbol, qsfLevelOne );

	try
	{
		m_bGotQuote = false;
		HRESULT hResult = m_pQuotes->AdviseSymbol(m_pQNotify, pszSymbol, qsfLevelOne);
		if( FAILED(hResult) )
			printf("AdviseSymbol() %s FAILED with hResult %d!\n", pszSymbol, hResult);
		else
			printf("AdviseSymbol() %s Succeeded.\n", pszSymbol);
		return SUCCEEDED(hResult);
	}
	catch( _com_error e )
	{
		printf("Could not AdviseSymbol %s - %s!\n", pszSymbol, e.ErrorMessage());
		return false;
	}
}

HRESULT CConsoleQuotes::OnQuoteData(QUOTERECORD* pQuote)
{
	// In a real app, this might just call into another object or something... for this
	// example, we'll just copy pertinent quote data to our local quote structure and
	// set our flag so main() can interrogate us and extract the data we stuffed.
	// The first record may or may not contain price data... it MAY only contain
	// fundamental information.

	COleDateTime cTime(pQuote->UTCDateTime);
	m_lastQuote.sTime = cTime.Format("%H:%M:%S");
//	m_lastQuote.sTime = pQuote->bstrTime;
	m_lastQuote.dLast = pQuote->dLast;
	m_lastQuote.dBid = pQuote->dBid;
	m_lastQuote.dAsk = pQuote->dAsk;
	m_lastQuote.lVol = pQuote->lVolume;
	m_lastQuote.eTick = (pQuote->tick == 1 ? "v" : "^");
	m_bGotQuote = true;
	return S_OK;
}

// We MUST define all callbacks, even though we're not using all of them:
HRESULT CConsoleQuotes::OnLevel2Data(LEVEL2RECORD* pQuote) { return S_OK; }
HRESULT CConsoleQuotes::OnTSData(TSRECORD* pQuote) { return S_OK; }
HRESULT CConsoleQuotes::OnOptionsData(OPTIONSRECORD* pQuote) { return S_OK; }

