/***********************************************************
 By using this material you agree to comply with the terms
 governing the MBTI systems and with the MBTI Site Terms and
 Conditions available for view at http://www.mbtrading.com/
 Copyright 2003 MB Trading, Inc. ALL RIGHTS RESERVED.
************************************************************/

// Quotes.cpp : Defines the entry point for the console application.

#include "stdafx.h"
#include "CConsoleQuotes.h"
#include "resource.h"

int main(int argc, char* argv[])
{
	IMbtComMgrPtr m_pComMgr;
	IMbtOrderClientPtr m_pOrderClient;
	IMbtAccountsPtr m_pAccounts;
	IMbtAccountPtr m_pAccount;
	IMbtAccountPtr m_pDefaultAccount;
	IMbtQuotesPtr m_pQuotes;
	HRESULT hResult;
	bool bOLEinited = false;

	// The following hack simply allows the use of the string table:
	HINSTANCE hInstance = (HINSTANCE)0x400000;
	AFX_MODULE_STATE* pModuleState = AfxGetModuleState();
	pModuleState->m_hCurrentInstanceHandle = hInstance;
	pModuleState->m_hCurrentResourceHandle = hInstance;

	// From an interface perspective, our objects are singlethreaded. COM will,
	// behind the scenes, create and marshal through a proxy object if multithreading
	// is attempted (COINIT_MULTITHREADED), which is not desired. Our quote callback
	// interface always expects to call back through a real object pointer, so we
	// MUST use single threaded apartment to create and access our objects.

	printf( "\n" );
	try // One great big try-catch block!
	{
		if( FAILED(hResult = CoInitializeEx(NULL, COINIT_APARTMENTTHREADED)) )
		{
			throw( (UINT)IDS_COINITFAILED );
		}
		else
		{
			bOLEinited = true;
			printf( "CoInitializeEx succeeded!\n" );
		}

		AfxOleInit();

		if( FAILED(m_pComMgr.CreateInstance (( __uuidof(MbtComMgr) )) ))
		{
			throw( (UINT)IDS_COMMGRFAILED );
		}
		else
		{
			printf( "MbtComMgr instantiation successful!\n" );
		}

		// Please enter your assigned SDK client ID into the Host ID below. You can
		// also enter your Username / Password below to circumvent the Login dialog box.
		long lHostId = 9;
		_bstr_t bstrUserName = "";
		_bstr_t bstrPassword = "";
		_bstr_t bstrHostVersion = "0";

		if( m_pComMgr->DoLogin(lHostId, bstrUserName, bstrPassword, bstrHostVersion) )
		{
			printf("DoLogin() succeeded!\n");
		}
		else
		{
			throw( (UINT)IDS_DOLOGINFAILED );
		}

		// Hide the splash right away so we can see what we're debugging
		HWND hWndSplash = ::FindWindow( NULL, "MbtNavSplash" );
		if( hWndSplash )
			::ShowWindow( hWndSplash, SW_HIDE );


		m_pOrderClient = m_pComMgr->OrderClient;

		// Spin a message loop so the OrderClient object can complete its login
		MSG msg;
		int iMaxWait = 10000;  // Timeout for login
		DWORD dwStart = GetTickCount();
		printf("Waiting %dms for OrderClient to finish loading accounts ", iMaxWait);
		while( GetMessage( &msg, NULL, 0, 0 ) )
		{
			TranslateMessage( &msg );
			DispatchMessage( &msg );

			if( GetTickCount() > dwStart + iMaxWait )
			{
				throw( (UINT)IDS_GOTINITLOGONSUCCEEDFAILED );
			}
			if( m_pOrderClient->GotInitialLogonSucceed() ) {
				DWORD dwEnd = GetTickCount();
				printf("\nAccounts loaded in %.4f seconds\n", (double(dwEnd) - double(dwStart)) / 1000);
				break;
			}
			printf(".");
		}

		printf("\n\n");
		printf("OrderClient -------------------------\n");
		printf(" Remote address  : %s\n", (char*)m_pOrderClient->RemoteAddress);
		printf(" Remote port     : %d\n", m_pOrderClient->RemotePort);
		printf(" UserName        : %s\n", (char*)m_pOrderClient->UserName);

		m_pAccounts = m_pOrderClient->Accounts;
		int iCount = m_pAccounts->Count;
		printf("\n" );
		printf("We have %d account(s) ----------------\n", iCount);
		for( int i = 0; i < iCount; i++ )
		{
			m_pAccount = m_pAccounts->Item[i];
			m_pAccount->Load();
			printf(" %s - %s\n", (char*)m_pAccount->Account, (char*)m_pAccount->Customer);
		}

		m_pDefaultAccount = m_pAccounts->DefaultAccount;
		printf("\n");
		if( m_pDefaultAccount )
		{
			printf("DefaultAccount ----------------------\n");
			printf(" Account         : %s - %s\n",
					(char*)m_pDefaultAccount->Account,
					(char*)m_pDefaultAccount->Customer);
			printf(" Current BP      : %12.2f\n", m_pDefaultAccount->CurrentBP);
			printf(" CurrentEquity   : %12.2f\n", m_pDefaultAccount->CurrentEquity);
			printf(" MorningBP       : %12.2f\n", m_pDefaultAccount->MorningBP );
			printf(" MorningCash     : %12.2f\n", m_pDefaultAccount->MorningCash );
			printf(" MorningEquity   : %12.2f\n", m_pDefaultAccount->MorningEquity );
			printf(" MornOvernightBP : %12.2f\n", m_pDefaultAccount->MornOvernightBP );
		}
		else
			throw( (UINT)IDS_NODEFAULTACCOUNT );

		m_pQuotes = m_pComMgr->Quotes;

		printf("\n");
		printf("Quotes ------------------------------\n");
		printf(" Remote Address  : %s\n", (char*)m_pQuotes->RemoteAddress );
		printf(" Remote Port     : %d\n", (char*)m_pQuotes->RemotePort );
		printf(" Username        : %s\n\n", (char*)m_pQuotes->UserName );

		// Set up our callback object and do the request.
		// Intentional stackframe so our qn goes out of scope and releases
		// everything it holds a reference to BEFORE main() ends

		CConsoleQuotes qn(m_pQuotes);
		printf("Advising symbol...\n");
		if( qn.DoAdvise( _T("MSFT") ) )
		{
			static int iMaxCBs = 9;    // callbacks limit
			int iMaxTime = 5000;       // milliseconds limit
			printf("\n");
			printf("Process callbacks until %dms timeout or %d callbacks:\n\n", iMaxTime, iMaxCBs);
			// Hang out pump message loop subject to a timeout and a number of callbacks
			// to get back a quote
			DWORD dwStart = GetTickCount();
			while( GetMessage( &msg, NULL, 0, 0 ) )
			{
				TranslateMessage( &msg );
				DispatchMessage( &msg );

				if( GetTickCount() > dwStart + iMaxTime ) // set a timeout
				{
					printf("Timeout hit\n\n");
					break;
				}
				if( qn.GotQuote() )  // if we have a callback...
				{
					// ...process it, reset our flag, and check max callback counter.
					const CurrentQuote& qt = qn.GetLastQuote();
					printf("%d.  %s  bid %g  ask %g  last %g  vol %d  tick %s\n",
						iMaxCBs, qt.sTime, qt.dBid, qt.dBid, qt.dAsk, qt.lVol, qt.eTick);
					qn.SetQuoteWaiting();
					iMaxCBs--;
					if( 1 > iMaxCBs )
					{
						printf("Max callbacks received\n\n");
						break;
					}
				}
			}
		}
		else
			throw( (UINT)IDS_DOADVISEFAILED );

		//	Now let's submit a simple order!
		printf("Now submitting an order; m_pOrderClient->Submit()\n");
		long lBuySell = VALUE_BUY;
		long lVol = 100;
		CString sSym = "MSFT";
		// Arbitrary price. Should be filled in with the
		//  last price for all Market and Stop orders
		double dPrice = 25.75;
		long lTIF = VALUE_DAY;
		long lOrdType = VALUE_LIMIT;
		long lVolType = VALUE_NORMAL;
		CString sRoute = "MBTX";
		BSTR bstrRetMsg = NULL; // NULL works, but still need to free below ::SysAllocString(L"");

		CString sOrder;
		sOrder.Format("Buy %d %s at $%.2f on %s\n", lVol, sSym, dPrice, sRoute);
		printf(sOrder);
		COleDateTime dDte;

		VARIANT_BOOL bSubmit = m_pOrderClient->Submit(
			lBuySell,
			lVol,
			_bstr_t(sSym),
			dPrice,
			0,
			lTIF,
			0,
			lOrdType,
			lVolType,
			0,
			m_pDefaultAccount,
			_bstr_t(sRoute),
			"", 0, 0, dDte, dDte, 0, 0, 0, 0, 0,
			&bstrRetMsg
		);

		if( bSubmit )
		{
			printf("Order submission successful! bstrRetMsg = [%S]\n", bstrRetMsg); // wide string!
		}
		else
		{
			printf("Order submission failed! bstrRetMsg = [%S]\n", bstrRetMsg); // wide string!
		}
		::SysFreeString(bstrRetMsg);
	}
	catch( UINT sResID ) {
		CString sMsg;
		sMsg.LoadString(sResID);
		printf("%s\n", sMsg);
	}

/*
	Clean up.

	Although we are using smart pointers here, we must explicitly cause them
	to Release() their underlying COM interface pointers here, rather than
	letting them do it themselves in their destructors after we return from main().

	The reason for this is that we're calling CoUnitialize() here, and as
	soon as that occurs, those interface pointers are no longer valid and
	will most likely crash when their destructors execute as they try to
	Release() an interface pointer that is now pointing to freed memory.
*/

	m_pQuotes = NULL;
	m_pDefaultAccount = NULL;
	m_pAccount = NULL;
	m_pAccounts = NULL;
	m_pOrderClient = NULL;
	m_pComMgr = NULL;

	if( bOLEinited )
	{
		printf("Calling CoUninitialize()");
		CoUninitialize();
	}
	printf("\nHit ENTER key to EXIT...");
	getchar();
	return S_OK;
}


